import 'package:fancywork/src/entities/lab.dart';
import 'package:fancywork/src/entities/palette_type.dart';
import 'package:fancywork/src/entities/rgba.dart';
import 'package:fancywork/src/utils/helpers.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';

class Dmc extends PaletteType {
  Dmc({
    @required String name,
    @required String code,
    @required Rgba rgba,
    @required Lab lab,
  })  : assert(name != null),
        assert(code != null),
        assert(rgba != null),
        assert(lab != null),
        super(
          name: name,
          code: code,
          rgba: rgba,
          lab: lab,
        );

  factory Dmc.fromJson(Map<String, dynamic> json) {
    return Dmc(
      name: json['name'],
      code: json['code'],
      rgba: json['rgba'] != null ? Rgba.fromJson(json['rgba']) : null,
      lab: json['lab'] != null ? Lab.fromJson(json['lab']) : null,
    );
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = {};

    data['name'] = name;
    data['code'] = code;
    if (rgba != null) {
      data['rgba'] = rgba.toJson();
    }
    if (lab != null) {
      data['lab'] = lab.toJson();
    }

    return data;
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is Dmc &&
          runtimeType == other.runtimeType &&
          name == other.name &&
          code == other.code &&
          rgba == other.rgba &&
          lab == other.lab;

  @override
  int get hashCode => hashValues(name, code, rgba, lab);

  @override
  toString() {
    final lab = rgbaToLab(rgba);

    return '''
    Dmc(
      name: "$name",
      code: "$code",
      rgba: $rgba,
      lab: $lab,
    )
    ''';
  }
}
