import 'package:fancywork/palette.dart';
import 'package:fancywork/src/entities/lab.dart';
import 'package:fancywork/src/entities/rgba.dart';
import 'package:fancywork/src/palette/dmc_values.dart';
import 'package:fancywork/src/utils/helpers.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:matcher/matcher.dart';

transformerTest() {
  group('transformer', () {
    test('lab', () {
      // 'Gray Green Dark',
      // '3768',
      // rgb: '{"B": 127, "G": 127, "R": 101}',
      // lab: '{"A": -9.30291, "B": -3.09718, "L": 51.2415}',
      // hsl: '{"H": 180, "L": 44.7058826684952, "S": 11.4035077393055}')
      Rgba rgba = Rgba(
        r: 101,
        g: 127,
        b: 127,
      );

      final lab = rgbaToLab(rgba);

      expect(lab.l.round(), 51, reason: 'L: ${lab.l.floor()}');
      expect(lab.a.round(), -9, reason: 'A: ${lab.a.floor()}');
      expect(lab.b.round(), -3, reason: 'B: ${lab.b.floor()}');
    });

    // dmc
    test('dmc', () {
      final dmc = kDmcValues;

      expect(dmc.length, 453, reason: 'dmc values len: ${dmc.length}');
    });

    // delta E
    test('delta E all zeros', () {
      Lab lab1 = Lab(l: 0, a: 0, b: 0);
      Lab lab2 = Lab(l: 0, a: 0, b: 0);

      final deltaE = getDeltaE(lab1, lab2);

      expect(deltaE, 0);
    });

    test('delta E l', () {
      Lab lab1 = Lab(l: 2, a: 0, b: 0);
      Lab lab2 = Lab(l: 0, a: 0, b: 0);

      expect(getDeltaE(lab1, lab2), 2);
      expect(getDeltaE(lab2, lab1), 2);
    });

    test('delta E a', () {
      Lab lab1 = Lab(l: 0, a: 2, b: 0);
      Lab lab2 = Lab(l: 0, a: 0, b: 0);

      expect(getDeltaE(lab1, lab2), 2);
      expect(getDeltaE(lab2, lab1), 2);
    });

    test('delta E b', () {
      Lab lab1 = Lab(l: 0, a: 0, b: 2);
      Lab lab2 = Lab(l: 0, a: 0, b: 0);

      expect(getDeltaE(lab1, lab2), 2);
      expect(getDeltaE(lab2, lab1), 2);
    });

    test('find dmc color', () {
      final black = findDmcByCode('Black');

      expect(black.name, '310');
    });

    test('find non-existing dmc color', () {
      final black = findDmcByCode('BlackTest');

      expect(black, null);
    });

    test('find dmc color exception', () {
      expect(
        () => findDmcByCode(null),
        throwsA(const TypeMatcher<ArgumentError>()),
      );
    });

    test('find nearest black color', () {
      final dmc = kDmcValues;
      Lab lab = Lab(l: 0, a: 0, b: 0);

      final Dmc black = findNearestPalette(lab, dmc);
      expect(black.name, '310');
    });

    test('rgbaToLab null params exception', () {
      expect(
        () => rgbaToLab(null),
        throwsA(const TypeMatcher<ArgumentError>()),
      );
    });

    test('getAvgColor null params exception', () {
      expect(
        () => getAvgColor(null),
        throwsA(const TypeMatcher<ArgumentError>()),
      );
    });

    test('findNearestPalette null params exception', () {
      expect(
        () => findNearestPalette(null, []),
        throwsA(const TypeMatcher<ArgumentError>()),
      );

      expect(
        () => findNearestPalette(Lab(l: 1, a: 1, b: 1), null),
        throwsA(const TypeMatcher<ArgumentError>()),
      );
    });

    test('getDeltaE null params exception', () {
      expect(
        () => getDeltaE(null, Lab(l: 1, a: 1, b: 1)),
        throwsA(const TypeMatcher<ArgumentError>()),
      );

      expect(
        () => getDeltaE(Lab(l: 1, a: 1, b: 1), null),
        throwsA(const TypeMatcher<ArgumentError>()),
      );
    });
  });
}
